import ssl
import requests
import json
from ruxit.api.base_plugin import RemoteBasePlugin
import logging
import atexit
import socket
from pyVim.connect import SmartConnect, Disconnect
from pyVmomi import vmodl
from pyVmomi import vim
from tools import alarm
from tools import cli
from datetime import datetime, timedelta
import fnmatch
import time

logger = logging.getLogger(__name__)

MAX_RETRIES = 3

class EncodeText(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, (bytes, bytearray)):
            return obj.decode('ASCII')
        # Let the base class default method raise the TypeError
        return json.JSONEncoder.default(self, obj)
        
class vCenterAlarmsPluginRemote(RemoteBasePlugin):
    
    previousEvents = []
    previous_alarms = {}
    events = 0
    
    # Upon first load, initialize objects, vars
    def initialize(self, **kwargs):
        # Store metrics that need to be compared with next polling interval
        self.poll_count = 1

        self.first_time = True
        
    def query(self, **kwargs):
        self.config = kwargs['config']
        user = self.config['auth_user']
        password = self.config['auth_password']
        server = self.config['vserver'].strip()
        port = 443 if self.config['port'] == '' else self.config['port']
        v_group = self.config['group'] if self.config['group'] != '' else 'Default group'
        tenant = self.config['api_url']
        api_token = self.config['api_token']
        
        self.polling_interval = int(self.config['polling_interval'])

        if self.poll_count < self.polling_interval and not self.first_time:
            self.poll_count += 1
            logger.info(f'... only polling every {self.polling_interval} minutes. Skipping this minute.')
            return
        self.poll_count = 1
        
        logger.info("Polling to VCenter started")
        
        proxy = self.build_proxy_url()
        
        if not tenant.endswith('/'):
            tenant = tenant + '/'
            
        if not tenant.startswith('http'):
            tenant = 'https://' + tenant
        
        if 'debug' in self.config and self.config['debug']:
            logging.basicConfig(level=logging.DEBUG)
            logger.setLevel(logging.DEBUG)
        else:
            logging.basicConfig(level=logging.INFO)
            logger.setLevel(logging.INFO)
        
        warning_alarms = self.config['warning_filter'].split(',')
        warning_alarms = [x.strip().lower() for x in warning_alarms]
        critical_alarms = self.config['critical_filter'].split(',')
        critical_alarms = [x.strip().lower() for x in critical_alarms]
        
        start_time = int(time.time()*1000)
        
        instance_name = ''
        service_instance = None
        
        # Creates the group and each custom device (vcenter). This is for visualization.
        group = self.topology_builder.create_group(f'vCenter - {v_group}', f'vCenter - {v_group}')
        device = group.create_element(f'{server}', f'{server}')
        device.report_property("Plugin version", kwargs["json_config"]["version"])
        
        if server not in self.previous_alarms:
            self.previous_alarms[server] = {}
        
        # SmartConnect uses sockets so let's set timeout before connecting.
        socket.setdefaulttimeout(10)
        
        try:
            context = ssl._create_unverified_context()
            
            # Change to SSL if needed
            service_instance = self.do_connect(server, user,password,port, context)

            # Ensure proper disconnection if RPM is terminated
            atexit.register(Disconnect, service_instance)
            
            for prop in service_instance.content.setting.setting:
                # Get the instance name
                if prop.key == 'VirtualCenter.InstanceName':
                    instance_name = prop.value
            
            device.absolute(key='availability', value=100)
            
            # Get all triggered alarms
            logger.debug('Getting alarms from VMWare API')
            alarms = service_instance.content.rootFolder.triggeredAlarmState
            alarm_count = len(alarms)
            logger.info(f'Found {alarm_count} alarms')
            device.absolute(key='total_alarms', value=alarm_count)
            
            # Iterate through each alarm
            logger.info('Matching alarms to filters')
            for alarm in alarms:
                if alarm.key not in self.previous_alarms[server] or (alarm.key in self.previous_alarms[server] and self.previous_alarms[server][alarm.key] == 1):
                    alarm_name = alarm.alarm.info.name
                    logger.debug(f'Alarm: {alarm}')
                    logger.debug(f"Alarm info: entityName={alarm.entity.name}, name={alarm_name}, status={alarm.overallStatus}")
                    parent = alarm.entity.parent
                    location = alarm.entity.name
                    
                    # Let's get recursively all its parents to know its location
                    while hasattr(parent, 'name'):
                        #logger.debug(f'Alarm parent: {parent.name}')
                        location = parent.name + '/' + location
                        parent = parent.parent
                    
                    logger.debug(f'Location: {location}')
                    

                    # Only send the alarms that match the filter
                    severity = None
                    if any(fnmatch.fnmatch(alarm_name.lower(), pattern) for pattern in warning_alarms):
                        severity = 'RESOURCE_CONTENTION'
                    elif any(fnmatch.fnmatch(alarm_name.lower(), pattern) for pattern in critical_alarms):
                        severity = 'AVAILABILITY_EVENT'
                    
                    props = {
                        'vCenter instance': instance_name,
                        'Triggered entity': alarm.entity.name,
                        'Alarm definition': alarm_name,
                        'Status': alarm.overallStatus.upper(),
                        'Location': location,
                        'Triggered time': datetime.fromtimestamp(int(alarm.time.timestamp())).strftime("%Y-%m-%d %H:%M:%S")
                    }
                    
                    if severity:
                        # The entity id is a hex conversion of the internal ID
                        entityId = f'CUSTOM_DEVICE-{device.id:X}'
                        logger.debug(f'EntityId = {entityId}')
                        api_url = f'{tenant}api/v1/events'
                        api_headers = {'Content-Type': 'application/json', 'Authorization': f'Api-Token {api_token}'}
                        api_body = {'eventType': severity,
                                    'timeoutMinutes': self.polling_interval+2,
                                    'attachRules': {
                                        'entityIds': [entityId]
                                    },
                                    'source': 'VCenter Extension',
                                    'title': f'{alarm_name} on {alarm.entity.name}',
                                    'description': f'{alarm_name} on {alarm.entity.name} is in {alarm.overallStatus} status. \n\nLocation: {location}',
                                    'customProperties': props,
                                    'allowDavisMerge': False
                                }
                        # The body gets built weird with single quotes and requests won't accept it even if sending it as json.
                        # So we re-encode it
                        api_body=json.dumps(api_body, cls=EncodeText )
                        logger.debug(f'Sending alarm to {entityId} with body {api_body}')
                        logger.debug(f'api_url: {api_url}')
                        
                        # Send the request to the API
                        r = requests.post(api_url, proxies=proxy, data=api_body, headers=api_headers, verify=False, timeout=10)

                        if r.status_code == 200:
                            logger.debug(f'Sent event successfully: {alarm_name} on {alarm.entity.name}')
                            self.previous_alarms[server][alarm.key] = 3
                        else:
                            device.report_error_event(f'Error sending VCenter triggered alarms to Dynatrace. {r.reason}', 'Error sending VCenter extension data')
                            logger.error(f'Error sending event to Dynatrace. Error code: {r.status_code}. Reason: {r.reason}. Data body: {api_body}')
                else:
                    self.previous_alarms[server][alarm.key] = self.previous_alarms[server][alarm.key]-1
            
            # If the any previous alarm does not exist in the current batch of alarms, remove it.
            for key in list(self.previous_alarms[server].keys()):
                if not any(key in alarm.key for alarm in alarms):
                    del self.previous_alarms[server][key]
                    logger.debug(f'{key} no longer in triggered alarms. Removing from previous alarms ...')
            
            self.first_time = False
            
        except vmodl.MethodFault as e:
            logger.error(f'Error occurred collecting data from VCenter: {e}')
            if not service_instance:
                logger.debug('Setting availability to 0')
                device.absolute(key='availability', value=0)
                
            device.report_error_event(f'Error occurred getting data from VCenter triggered alarms. Msg: {e.msg}', 'Error collecting data from VCenter extension')
        finally:
            if service_instance:
                Disconnect(service_instance)
                logger.info('Disconnected')
               
    # Method to build proxy list according to config fields
    def build_proxy_url(self):
        proxy_address = self.config.get("proxy_address")
        proxy_username = self.config.get("proxy_username")
        proxy_password = self.config.get("proxy_password")

        if proxy_address:
            protocol, address = proxy_address.split("://")
            proxy_url = f"{protocol}://"
            if proxy_username:
                proxy_url += proxy_username
            if proxy_password:
                proxy_url += f":{proxy_password}"
            proxy_url += f"@{address}"
            return {"https": proxy_url}

        return {}
               
    def do_connect(self, server, user,password,port, context):
        for _ in range(MAX_RETRIES):
            try:
                service_instance = SmartConnect(host=server, user=user,pwd=password,port=port, sslContext=context)
                break
            except socket.timeout:
                logger.error('Tried to connect for 10 seconds but vCenter did not respond. Trying again now...')
        else:
            raise Exception(f"Tried to connect {MAX_RETRIES} times but vCenter did not respond, giving up until the next minute.")
            
        if service_instance:
            return service_instance